<?php

declare(strict_types=1);

namespace Drupal\role_enum\Entity\Trait;

use Drupal\role_enum\Enum\ConfigBackedRoleInterface;
use Drupal\role_enum\Enum\RoleInterface as RoleEnumInterface;
use Drupal\role_enum\Exception\MustBeConfigBackedRoleException;

/**
 * A trait to be used on user entity bundle classes.
 *
 * Implements the methods at \Drupal\role_enum\Entity\Trait\RoleTrait.
 *
 * The user entity bundle should also implement the companion interface.
 *
 * @see \Drupal\role_enum\Entity\RoleInterface
 * @property \Drupal\Core\Field\EntityReferenceFieldItemListInterface<\Drupal\user\RoleInterface> $roles
 */
trait RoleTrait {

  /**
   * @phpstan-return $this
   */
  public function roleEnumAdd(RoleEnumInterface ...$roles) {
    foreach ($roles as $role) {
      if (!$role instanceof ConfigBackedRoleInterface) {
        throw MustBeConfigBackedRoleException::create($role::class);
      }

      $this->roles->appendItem($role->id());
    }

    return $this;
  }

  /**
   * @phpstan-return $this
   */
  public function roleEnumRemove(RoleEnumInterface ...$roles) {
    foreach ($roles as $role) {
      if (!$role instanceof ConfigBackedRoleInterface) {
        throw MustBeConfigBackedRoleException::create($role::class);
      }

      foreach ($this->roles as $delta => $value) {
        if ($value->target_id === $role->id()) {
          $this->roles->removeItem($delta);
        }
      }
    }

    return $this;
  }

  public function roleEnumHas(RoleEnumInterface $role): bool {
    if (!$role instanceof ConfigBackedRoleInterface) {
      throw MustBeConfigBackedRoleException::create($role::class);
    }

    return \in_array(
      $role->id(),
      // @phpstan-ignore-next-line
      \array_column($this->roles->getValue(), 'target_id'),
      TRUE,
    );
  }

}
