<?php

declare(strict_types=1);

namespace Drupal\Tests\role_enum\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\role_enum_test\Entity\RoleEnumTestUser;
use Drupal\role_enum_test\Enum\TestRoles;
use Drupal\user\Entity\Role;
use Drupal\user\Entity\User;

/**
 * Tests Role Enum property and trait.
 */
final class RoleEnumTest extends KernelTestBase {

  protected static $modules = [
    'user',
    'bca',
    'role_enum',
    'role_enum_test',
  ];

  protected function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('user');
    Role::create(['id' => TestRoles::Foo->value, 'label' => 'Test'])->setSyncing(TRUE)->save();
    Role::create(['id' => TestRoles::Bar->value, 'label' => 'Test'])->setSyncing(TRUE)->save();
  }

  public function testProperty(): void {
    $user = User::create(['name' => $this->randomMachineName()]);
    $user->save();
    static::assertInstanceOf(RoleEnumTestUser::class, $user);

    static::assertFalse($user->r->has(TestRoles::Foo));
    $user->r->add(TestRoles::Foo);
    static::assertTrue($user->r->has(TestRoles::Foo));
    $user->r->remove(TestRoles::Foo);
    static::assertFalse($user->r->has(TestRoles::Foo));

    // Chained.
    static::assertTrue($user->r->add(TestRoles::Foo)->r->has(TestRoles::Foo));
  }

  public function testTrait(): void {
    $user = User::create(['name' => $this->randomMachineName()]);
    $user->save();
    static::assertInstanceOf(RoleEnumTestUser::class, $user);

    static::assertFalse($user->roleEnumHas(TestRoles::Foo));
    $user->roleEnumAdd(TestRoles::Foo);
    static::assertTrue($user->roleEnumHas(TestRoles::Foo));
    $user->roleEnumRemove(TestRoles::Foo);
    static::assertFalse($user->roleEnumHas(TestRoles::Foo));

    // Chained.
    static::assertTrue($user->roleEnumAdd(TestRoles::Foo)->roleEnumHas(TestRoles::Foo));
  }

}
